@@ -12,17 +12,22 @@
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#include "lib.h"
-#include "segtype.h"
-#include "display.h"
-#include "text_export.h"
-#include "config.h"
-#include "str_list.h"
-#include "targets.h"
-#include "lvm-string.h"
-#include "activate.h"
-#include "metadata.h"
-#include "lv_alloc.h"
+#include "base/memory/zalloc.h"
+#include "lib/misc/lib.h"
+#include "lib/metadata/segtype.h"
+#include "lib/display/display.h"
+#include "lib/format_text/text_export.h"
+#include "lib/config/config.h"
+#include "lib/datastruct/str_list.h"
+#include "lib/activate/targets.h"
+#include "lib/misc/lvm-string.h"
+#include "lib/activate/activate.h"
+#include "lib/metadata/metadata.h"
+#include "lib/metadata/lv_alloc.h"
+
+static int _raid_target_present(struct cmd_context *cmd,
+				const struct lv_segment *seg __attribute__((unused)),
+				unsigned *attributes);
 
 static void _raid_display(const struct lv_segment *seg)
 {
@@ -235,22 +240,6 @@ static int _raid_text_export(const struct lv_segment *seg, struct formatter *f)
 	return _raid_text_export_raid(seg, f);
 }
 
-static int _raid_target_status_compatible(const char *type)
-{
-	return (strstr(type, "raid") != NULL);
-}
-
-static void _raid_destroy(struct segment_type *segtype)
-{
-	dm_free((void *) segtype->dso);
-	dm_free(segtype);
-}
-
-#ifdef DEVMAPPER_SUPPORT
-static int _raid_target_present(struct cmd_context *cmd,
-				const struct lv_segment *seg __attribute__((unused)),
-				unsigned *attributes);
-
 static int _raid_add_target_line(struct dev_manager *dm __attribute__((unused)),
 				 struct dm_pool *mem __attribute__((unused)),
 				 struct cmd_context *cmd __attribute__((unused)),
@@ -368,6 +357,18 @@ static int _raid_add_target_line(struct dev_manager *dm __attribute__((unused)),
 	return add_areas_line(dm, seg, node, 0u, seg->area_count);
 }
 
+static int _raid_target_status_compatible(const char *type)
+{
+	return (strstr(type, "raid") != NULL);
+}
+
+static void _raid_destroy(struct segment_type *segtype)
+{
+	free((void *) segtype->dso);
+	free(segtype);
+}
+
+#ifdef DEVMAPPER_SUPPORT
 static int _raid_target_percent(void **target_state,
 				dm_percent_t *percent,
 				struct dm_pool *mem,
@@ -468,63 +469,6 @@ static int _check_feature(const struct raid_feature *feature, uint32_t maj, uint
 	       (maj == feature->maj && min == feature->min && patchlevel >= feature->patchlevel);
 }
 
-/* Check availability of raid10 taking data copies into consideration. */
-static bool _raid10_is_available(const struct logical_volume *lv)
-{
-	uint32_t i, rebuilds_per_group, s;
-	const uint32_t copies = 2; /* FIXME: we only support 2-way mirrors (i.e. 2 data copies) in RAID10 for now. */
-	struct lv_segment *seg = first_seg(lv); /* We only have one segment in RaidLVs for now. */
-
-	for (i = 0; i < seg->area_count * copies; ++i) {
-		s = i % seg->area_count;
-
-		if (!(i % copies))
-			rebuilds_per_group = 0;
-
-		if (seg_type(seg, s) == AREA_LV &&
-		    (lv_is_partial(seg_lv(seg, s)) ||
-		     lv_is_virtual(seg_lv(seg, s))))
-			rebuilds_per_group++;
-
-		if (rebuilds_per_group >= copies)
-			return false;
-	}
-
-	return true;
-}
-
-/*
- * Return true in case RaidLV with specific RAID level is available.
- *
- * - raid0: all legs have to be live
- * - raid1 : minimum of 1 leg live
- * - raid4/5: maximum of 1 leg unavailable
- * - raid6:  maximum of 2 legs unavailable
- * - raid10: minimum of 1 leg per mirror group available
- *
- */
-bool raid_is_available(const struct logical_volume *lv)
-{
-	uint32_t s, missing_legs = 0;
-	struct lv_segment *seg = first_seg(lv); /* We only have one segment in RaidLVs for now. */
-
-	/* Be cautious about bogus calls. */
-	if (!seg || !seg_is_raid(seg))
-		return false;
-
-	if (seg_is_any_raid10(seg))
-		return _raid10_is_available(lv);
-
-	/* Count missing RAID legs */
-	for (s = 0; s < seg->area_count; ++s)
-		if (seg_type(seg, s) == AREA_LV &&
-		    lv_is_partial(seg_lv(seg, s)))
-			missing_legs++;
-
-	/* Degradation: segtype raid1 may miss legs-1, raid0/4/5/6 may loose parity devices. */
-	return missing_legs <= (seg_is_raid1(seg) ? seg->area_count - 1 : seg->segtype->parity_devs);
-}
-
 static int _raid_target_present(struct cmd_context *cmd,
 				const struct lv_segment *seg __attribute__((unused)),
 				unsigned *attributes)
@@ -550,8 +494,10 @@ static int _raid_target_present(struct cmd_context *cmd,
 	if (!_raid_checked) {
 		_raid_checked = 1;
 
-		if (!(_raid_present = target_present_version(cmd, TARGET_NAME_RAID, 1,
-							     &maj, &min, &patchlevel)))
+		if (!(_raid_present = target_present(cmd, TARGET_NAME_RAID, 1)))
+			return 0;
+
+		if (!target_version("raid", &maj, &min, &patchlevel))
 			return_0;
 
 		for (i = 0; i < DM_ARRAY_SIZE(_features); ++i)
@@ -623,9 +569,9 @@ static struct segtype_handler _raid_ops = {
 	.text_import_area_count = _raid_text_import_area_count,
 	.text_import = _raid_text_import,
 	.text_export = _raid_text_export,
+	.add_target_line = _raid_add_target_line,
 	.target_status_compatible = _raid_target_status_compatible,
 #ifdef DEVMAPPER_SUPPORT
-	.add_target_line = _raid_add_target_line,
 	.target_percent = _raid_target_percent,
 	.target_present = _raid_target_present,
 	.check_transient_status = _raid_transient_status,
@@ -672,7 +618,7 @@ static struct segment_type *_init_raid_segtype(struct cmd_context *cmd,
 					       const char *dso,
 					       uint64_t monitored)
 {
-	struct segment_type *segtype = dm_zalloc(sizeof(*segtype));
+	struct segment_type *segtype = zalloc(sizeof(*segtype));
 
 	if (!segtype) {
 		log_error("Failed to allocate memory for %s segtype",
@@ -686,7 +632,7 @@ static struct segment_type *_init_raid_segtype(struct cmd_context *cmd,
 
 	/* Never monitor raid0 or raid0_meta LVs */
 	if (!segtype_is_any_raid0(segtype) &&
-	    dso && (dso = dm_strdup(dso))) {
+	    dso && (dso = strdup(dso))) {
 		segtype->dso = dso;
 		segtype->flags |= monitored;
 	}
@@ -730,7 +676,7 @@ int init_multiple_segtypes(struct cmd_context *cmd, struct segtype_library *segl
 			break;
 		}
 
-	dm_free(dso);
+	free(dso);
 
 	return r;
 }
