@@ -15,9 +15,6 @@ test_description="ensure that 'vgreduce --removemissing' works on mirrored LV"
 
 SKIP_WITH_LVMPOLLD=1
 
-# disable lvmetad logging as it bogs down test systems
-export LVM_TEST_LVMETAD_DEBUG_OPTS=${LVM_TEST_LVMETAD_DEBUG_OPTS-}
-
 . lib/inittest
 
 list_pvs=()
