@@ -51,9 +51,6 @@ do { \
 	printf(fmt "\n", ##args); \
 } while (0)
 
-#define dm_malloc malloc
-#define dm_strdup strdup
-#define dm_free free
 #define dm_snprintf snprintf
 
 static int dm_strncpy(char *dest, const char *src, size_t n)
@@ -134,13 +131,13 @@ static inline int configtype_arg(struct cmd_context *cmd __attribute__((unused))
 #define ALLOW_UUID_AS_NAME      0x00000010
 #define LOCKD_VG_SH             0x00000020
 #define NO_METADATA_PROCESSING  0x00000040
-#define IGNORE_PERSISTENT_FILTER 0x00000080
 #define MUST_USE_ALL_ARGS        0x00000100
-#define NO_LVMETAD_AUTOSCAN      0x00000200
 #define ENABLE_DUPLICATE_DEVS    0x00000400
 #define DISALLOW_TAG_ARGS        0x00000800
 #define GET_VGNAME_FROM_OPTIONS  0x00001000
 #define CAN_USE_ONE_SCAN	 0x00002000
+#define ALLOW_HINTS              0x00004000
+
 
 /* create foo_CMD enums for command def ID's in command-lines.in */
 
@@ -367,7 +364,7 @@ static int _opt_str_to_num(struct command *cmd, char *str)
 	int i;
 	int first = 0, last = ARG_COUNT - 1, middle;
 
-	(void) dm_strncpy(long_name, str, sizeof(long_name));
+	dm_strncpy(long_name, str, sizeof(long_name));
 
 	if ((p = strstr(long_name, "_long")))
 		/*
@@ -746,7 +743,7 @@ static void _add_oo_definition_line(const char *name, const char *line)
 
 	oo = &_oo_lines[_oo_line_count++];
 
-	if (!(oo->name = dm_strdup(name))) {
+	if (!(oo->name = strdup(name))) {
 		log_error("Failer to duplicate name %s.", name);
 		return; /* FIXME: return code */
 	}
@@ -759,7 +756,7 @@ static void _add_oo_definition_line(const char *name, const char *line)
 	}
 
 	start = strchr(line, ':') + 2;
-	if (!(oo->line = dm_strdup(start))) {
+	if (!(oo->line = strdup(start))) {
 		log_error("Failer to duplicate line %s.", start);
 		return;
 	}
@@ -780,14 +777,14 @@ static void _append_oo_definition_line(const char *new_line)
 
 	/* +2 = 1 space between old and new + 1 terminating \0 */
 	len = strlen(old_line) + strlen(new_line) + 2;
-	line = dm_malloc(len);
+	line = malloc(len);
 	if (!line) {
 		log_error("Parsing command defs: no memory.");
 		return;
 	}
 
 	(void) dm_snprintf(line, len, "%s %s", old_line, new_line);
-	dm_free(oo->line);
+	free(oo->line);
 	oo->line = line;
 }
 
@@ -802,7 +799,7 @@ static char *_get_oo_line(const char *str)
 	char str2[OO_NAME_LEN];
 	int i;
 
-	(void) dm_strncpy(str2, str, sizeof(str2));
+	dm_strncpy(str2, str, sizeof(str2));
 	if ((end = strchr(str2, ':')))
 		*end = '\0';
 	if ((end = strchr(str2, ',')))
@@ -834,14 +831,14 @@ static void _include_optional_opt_args(struct cmd_context *cmdtool, struct comma
 		return;
 	}
 
-	if (!(line = dm_strdup(oo_line))) {
+	if (!(line = strdup(oo_line))) {
 		cmd->cmd_flags |= CMD_FLAG_PARSE_ERROR;
 		return;
 	}
 
 	_split_line(line, &line_argc, line_argv, ' ');
 	__add_optional_opt_line(cmdtool, cmd, line_argc, line_argv);
-	dm_free(line);
+	free(line);
 }
 
 /*
@@ -1090,14 +1087,14 @@ static void _include_required_opt_args(struct cmd_context *cmdtool, struct comma
 		return;
 	}
 
-	if (!(line = dm_strdup(oo_line))) {
+	if (!(line = strdup(oo_line))) {
 		cmd->cmd_flags |= CMD_FLAG_PARSE_ERROR;
 		return;
 	}
 
 	_split_line(line, &line_argc, line_argv, ' ');
 	_add_required_opt_line(cmdtool, cmd, line_argc, line_argv);
-	dm_free(line);
+	free(line);
 }
 
 /* Process what follows command_name, which are required opt/pos args. */
@@ -1465,11 +1462,10 @@ int define_commands(struct cmd_context *cmdtool, const char *run_name)
 
 		if (_is_desc_line(line_argv[0]) && !skip && cmd) {
 			char *desc = dm_pool_strdup(cmdtool->libmem, line_orig);
-			if (cmd->desc) {
+			if (cmd->desc && desc) {
 				int newlen = strlen(cmd->desc) + strlen(desc) + 2;
 				char *newdesc = dm_pool_alloc(cmdtool->libmem, newlen);
 				if (newdesc) {
-					memset(newdesc, 0, newlen);
 					snprintf(newdesc, newlen, "%s %s", cmd->desc, desc);
 					cmd->desc = newdesc;
 				} else {
@@ -1568,8 +1564,8 @@ int define_commands(struct cmd_context *cmdtool, const char *run_name)
 
 	for (i = 0; i < _oo_line_count; i++) {
 		struct oo_line *oo = &_oo_lines[i];
-		dm_free(oo->name);
-		dm_free(oo->line);
+		free(oo->name);
+		free(oo->line);
 	}
 	memset(&_oo_lines, 0, sizeof(_oo_lines));
 	_oo_line_count = 0;
@@ -1916,7 +1912,7 @@ void print_usage(struct command *cmd, int longhelp, int desc_first)
 			 * see print_common_options_cmd()
 			 */
 
-			if (cname && (cname->variants > 1) && cname->common_options[opt_enum])
+			if ((cname->variants > 1) && cname->common_options[opt_enum])
 				continue;
 
 			printf("\n\t[");
@@ -1956,7 +1952,7 @@ void print_usage(struct command *cmd, int longhelp, int desc_first)
 			 * see print_common_options_cmd()
 			 */
 
-			if (cname && (cname->variants > 1) && cname->common_options[opt_enum])
+			if ((cname->variants > 1) && cname->common_options[opt_enum])
 				continue;
 
 			printf("\n\t[");
@@ -2298,8 +2294,7 @@ static void _print_val_man(struct command_name *cname, int opt_enum, int val_enu
 	}
 
 	if (strchr(str, '|')) {
-		if (!(line = dm_strdup(str)))
-			return;
+		line = strdup(str);
 		_split_line(line, &line_argc, line_argv, '|');
 		for (i = 0; i < line_argc; i++) {
 			if (i)
@@ -2309,7 +2304,7 @@ static void _print_val_man(struct command_name *cname, int opt_enum, int val_enu
 			else
 				printf("\\fB%s\\fP", line_argv[i]);
 		}
-		dm_free(line);
+		free(line);
 		return;
 	}
 
@@ -3233,8 +3228,9 @@ static void _print_man_all_positions_desc(struct command_name *cname)
 	       "capitalization, e.g. 'k' and 'K' both refer to 1024.\n"
 	       "The default input unit is specified by letter, followed by |UNIT.\n"
 	       "UNIT represents other possible input units: \\fBbBsSkKmMgGtTpPeE\\fP.\n"
-	       "b|B is bytes, s|S is sectors of 512 bytes, k|K is KiB,\n"
-	       "m|M is MiB, g|G is GiB, t|T is TiB, p|P is PiB, e|E is EiB.\n"
+	       "b|B is bytes, s|S is sectors of 512 bytes, k|K is kilobytes,\n"
+	       "m|M is megabytes, g|G is gigabytes, t|T is terabytes,\n"
+	       "p|P is petabytes, e|E is exabytes.\n"
 	       "(This should not be confused with the output control --units, where\n"
 	       "capital letters mean multiple of 1000.)\n");
 
@@ -3318,7 +3314,7 @@ static int _include_description_file(char *name, char *des_file)
 		goto out_close;
 	}
 
-	if (!(buf = dm_malloc(statbuf.st_size + 1))) {
+	if (!(buf = malloc(statbuf.st_size + 1))) {
 		log_error("Failed to allocate buffer for description file %s.", des_file);
 		goto out_close;
 	}
@@ -3333,7 +3329,7 @@ static int _include_description_file(char *name, char *des_file)
 	r = 1;
 
 out_free:
-	dm_free(buf);
+	free(buf);
 out_close:
 	(void) close(fd);
 
@@ -3384,7 +3380,7 @@ static int _print_man(char *name, char *des_file, int secondary)
 
 		if (!prev_cmd || strcmp(prev_cmd->name, cmd->name)) {
 			printf(".SH NAME\n");
-			if (cname && cname->desc)
+			if (cname->desc)
 				printf("%s - %s\n", lvmname, cname->desc);
 			else
 				printf("%s\n", lvmname);
@@ -3520,7 +3516,7 @@ int main(int argc, char *argv[])
 
 	memset(&commands, 0, sizeof(commands));
 
-	if (!(stdout_buf = dm_malloc(sz)))
+	if (!(stdout_buf = malloc(sz)))
 		log_error("Failed to allocate stdout buffer; carrying on with default buffering.");
 	else
 		setbuffer(stdout, stdout_buf, sz);
@@ -3550,12 +3546,9 @@ int main(int argc, char *argv[])
 		goto out_free;
 	}
 
-	if (optind < argc) {
-		if (!(cmdname = strdup(argv[optind++]))) {
-			log_error("Out of memory.");
-			goto out_free;
-		}
-	} else {
+	if (optind < argc)
+		cmdname = strdup(argv[optind++]);
+	else {
 		log_error("Missing command name.");
 		goto out_free;
 	}
